@@ -12,6 +12,7 @@ import java.util.*;
 import org.apache.xerces.dom.CoreDocumentImpl;
 import org.jruby.Ruby;
 import org.jruby.RubyArray;
+import org.jruby.RubyBoolean;
 import org.jruby.RubyClass;
 import org.jruby.RubyFixnum;
 import org.jruby.RubyInteger;
@@ -35,6 +36,7 @@ import org.w3c.dom.NamedNodeMap;
 import org.w3c.dom.Node;
 import org.w3c.dom.NodeList;
 import org.w3c.dom.Text;
+import org.w3c.dom.Comment;
 
 import nokogiri.internals.HtmlDomParserContext;
 import nokogiri.internals.NokogiriHelpers;
@@ -303,6 +305,14 @@ public class XmlNode extends RubyObject
     IRubyObject name = args[0];
     IRubyObject doc = args[1];
 
+    if (!(doc instanceof XmlNode)) {
+      throw context.runtime.newArgumentError("document must be a Nokogiri::XML::Node");
+    }
+    if (!(doc instanceof XmlDocument)) {
+      // TODO: deprecate allowing Node
+      context.runtime.getWarnings().warn("Passing a Node as the second parameter to Node.new is deprecated. Please pass a Document instead, or prefer an alternative constructor like Node#add_child. This will become an error in a future release of Nokogiri.");
+    }
+
     Document document = asXmlNode(context, doc).getOwnerDocument();
     if (document == null) {
       throw context.runtime.newArgumentError("node must have owner document");
@@ -421,7 +431,14 @@ public class XmlNode extends RubyObject
     String nsURI = e.lookupNamespaceURI(prefix);
     this.node = NokogiriHelpers.renameNode(e, nsURI, e.getNodeName());
 
-    if (nsURI == null || nsURI.isEmpty()) { return; }
+    if (nsURI == null || nsURI.isEmpty()) {
+      RubyBoolean ns_inherit =
+        (RubyBoolean)document(context.runtime).getInstanceVariable("@namespace_inheritance");
+      if (ns_inherit.isTrue()) {
+        set_namespace(context, ((XmlNode)parent(context)).namespace(context));
+      }
+      return;
+    }
 
     String currentPrefix = e.getParentNode().lookupPrefix(nsURI);
     String currentURI = e.getParentNode().lookupNamespaceURI(prefix);
@@ -645,7 +662,7 @@ public class XmlNode extends RubyObject
 
     final XmlDocument doc = document(context.runtime);
     for (int i = 0; i < nodeMap.getLength(); i++) {
-      if ((doc instanceof HtmlDocument) || !NokogiriHelpers.isNamespace(nodeMap.item(i))) {
+      if ((doc instanceof Html4Document) || !NokogiriHelpers.isNamespace(nodeMap.item(i))) {
         attr.append(getCachedNodeOrCreate(runtime, nodeMap.item(i)));
       }
     }
@@ -811,8 +828,8 @@ public class XmlNode extends RubyObject
     XmlDocument document = document(runtime);
     if (document == null) { return context.nil; }
 
-    if (document instanceof HtmlDocument) {
-      klass = getNokogiriClass(runtime, "Nokogiri::HTML::Document");
+    if (document instanceof Html4Document) {
+      klass = getNokogiriClass(runtime, "Nokogiri::HTML4::Document");
       ctx = new HtmlDomParserContext(runtime, options);
       ((HtmlDomParserContext) ctx).enableDocumentFragment();
       ctx.setStringInputSource(context, str, context.nil);
@@ -824,7 +841,7 @@ public class XmlNode extends RubyObject
 
     // TODO: for some reason, document.getEncoding() can be null or nil (don't know why)
     // run `test_parse_with_unparented_html_text_context_node' few times to see this happen
-    if (document instanceof HtmlDocument && !(document.getEncoding() == null || document.getEncoding().isNil())) {
+    if (document instanceof Html4Document && !(document.getEncoding() == null || document.getEncoding().isNil())) {
       HtmlDomParserContext htmlCtx = (HtmlDomParserContext) ctx;
       htmlCtx.setEncoding(document.getEncoding().asJavaString());
     }
@@ -1148,7 +1165,7 @@ public class XmlNode extends RubyObject
   namespace(ThreadContext context)
   {
     final XmlDocument doc = document(context.runtime);
-    if (doc instanceof HtmlDocument) { return context.nil; }
+    if (doc instanceof Html4Document) { return context.nil; }
 
     String namespaceURI = node.getNamespaceURI();
     if (namespaceURI == null || namespaceURI.isEmpty()) {
@@ -1183,7 +1200,7 @@ public class XmlNode extends RubyObject
     // updated.
     final XmlDocument doc = document(context.runtime);
     if (doc == null) { return context.runtime.newEmptyArray(); }
-    if (doc instanceof HtmlDocument) { return context.runtime.newEmptyArray(); }
+    if (doc instanceof Html4Document) { return context.runtime.newEmptyArray(); }
 
     List<XmlNamespace> namespaces = doc.getNamespaceCache().get(node);
     return context.runtime.newArray((List) namespaces);
@@ -1199,7 +1216,7 @@ public class XmlNode extends RubyObject
   {
     final XmlDocument doc = document(context.runtime);
     if (doc == null) { return context.runtime.newEmptyArray(); }
-    if (doc instanceof HtmlDocument) { return context.runtime.newEmptyArray(); }
+    if (doc instanceof Html4Document) { return context.runtime.newEmptyArray(); }
 
     Node previousNode;
     if (node.getNodeType() == Node.ELEMENT_NODE) {
@@ -1335,7 +1352,7 @@ public class XmlNode extends RubyObject
   private boolean
   isHtmlDoc(ThreadContext context)
   {
-    return document(context).getMetaClass().isKindOfModule(getNokogiriClass(context.runtime, "Nokogiri::HTML::Document"));
+    return document(context).getMetaClass().isKindOfModule(getNokogiriClass(context.runtime, "Nokogiri::HTML4::Document"));
   }
 
   private boolean
@@ -1516,61 +1533,65 @@ public class XmlNode extends RubyObject
   {
     String type;
     switch (node.getNodeType()) {
-    case Node.ELEMENT_NODE:
-      if (this instanceof XmlElementDecl) {
-        type = "ELEMENT_DECL";
-      } else if (this instanceof XmlAttributeDecl) {
-        type = "ATTRIBUTE_DECL";
-      } else if (this instanceof XmlEntityDecl) {
-        type = "ENTITY_DECL";
-      } else {
-        type = "ELEMENT_NODE";
-      }
-      break;
-    case Node.ATTRIBUTE_NODE:
-      type = "ATTRIBUTE_NODE";
-      break;
-    case Node.TEXT_NODE:
-      type = "TEXT_NODE";
-      break;
-    case Node.CDATA_SECTION_NODE:
-      type = "CDATA_SECTION_NODE";
-      break;
-    case Node.ENTITY_REFERENCE_NODE:
-      type = "ENTITY_REF_NODE";
-      break;
-    case Node.ENTITY_NODE:
-      type = "ENTITY_NODE";
-      break;
-    case Node.PROCESSING_INSTRUCTION_NODE:
-      type = "PI_NODE";
-      break;
-    case Node.COMMENT_NODE:
-      type = "COMMENT_NODE";
-      break;
-    case Node.DOCUMENT_NODE:
-      if (this instanceof HtmlDocument) {
-        type = "HTML_DOCUMENT_NODE";
-      } else {
-        type = "DOCUMENT_NODE";
-      }
-      break;
-    case Node.DOCUMENT_TYPE_NODE:
-      type = "DOCUMENT_TYPE_NODE";
-      break;
-    case Node.DOCUMENT_FRAGMENT_NODE:
-      type = "DOCUMENT_FRAG_NODE";
-      break;
-    case Node.NOTATION_NODE:
-      type = "NOTATION_NODE";
-      break;
-    default:
-      return context.runtime.newFixnum(0);
+      case Node.ELEMENT_NODE:
+        if (this instanceof XmlElementDecl) {
+          type = "ELEMENT_DECL";
+        } else if (this instanceof XmlAttributeDecl) {
+          type = "ATTRIBUTE_DECL";
+        } else if (this instanceof XmlEntityDecl) {
+          type = "ENTITY_DECL";
+        } else {
+          type = "ELEMENT_NODE";
+        }
+        break;
+      case Node.ATTRIBUTE_NODE:
+        type = "ATTRIBUTE_NODE";
+        break;
+      case Node.TEXT_NODE:
+        type = "TEXT_NODE";
+        break;
+      case Node.CDATA_SECTION_NODE:
+        type = "CDATA_SECTION_NODE";
+        break;
+      case Node.ENTITY_REFERENCE_NODE:
+        type = "ENTITY_REF_NODE";
+        break;
+      case Node.ENTITY_NODE:
+        type = "ENTITY_NODE";
+        break;
+      case Node.PROCESSING_INSTRUCTION_NODE:
+        type = "PI_NODE";
+        break;
+      case Node.COMMENT_NODE:
+        type = "COMMENT_NODE";
+        break;
+      case Node.DOCUMENT_NODE:
+        if (this instanceof Html4Document) {
+          type = "HTML_DOCUMENT_NODE";
+        } else {
+          type = "DOCUMENT_NODE";
+        }
+        break;
+      case Node.DOCUMENT_TYPE_NODE:
+        type = "DOCUMENT_TYPE_NODE";
+        break;
+      case Node.DOCUMENT_FRAGMENT_NODE:
+        type = "DOCUMENT_FRAG_NODE";
+        break;
+      case Node.NOTATION_NODE:
+        type = "NOTATION_NODE";
+        break;
+      default:
+        return context.runtime.newFixnum(0);
     }
 
     return getNokogiriClass(context.runtime, "Nokogiri::XML::Node").getConstant(type);
   }
 
+  /*
+   * NOTE that the behavior of this function is very difference from the CRuby implementation, see
+   * the docstring in ext/nokogiri/xml_node.c for details.
+   */
   @JRubyMethod
   public IRubyObject
   line(ThreadContext context)
@@ -1578,7 +1599,10 @@ public class XmlNode extends RubyObject
     Node root = getOwnerDocument();
     int[] counter = new int[1];
     count(root, counter);
-    return RubyFixnum.newFixnum(context.runtime, counter[0] + 1);
+    // offset of 2:
+    // - one because humans start counting at 1 not zero
+    // - one to account for the XML declaration present in the output
+    return RubyFixnum.newFixnum(context.runtime, counter[0] + 2);
   }
 
   private boolean
@@ -1591,9 +1615,14 @@ public class XmlNode extends RubyObject
     NodeList list = node.getChildNodes();
     for (int jchild = 0; jchild < list.getLength(); jchild++) {
       Node child = list.item(jchild);
+      String text = null;
 
       if (child instanceof Text) {
-        String text = ((Text)child).getData();
+        text = ((Text)child).getData();
+      } else if (child instanceof Comment) {
+        text = ((Comment)child).getData();
+      }
+      if (text != null) {
         int textLength = text.length();
         for (int jchar = 0; jchar < textLength; jchar++) {
           if (text.charAt(jchar) == '\n') {
@@ -1676,23 +1705,23 @@ public class XmlNode extends RubyObject
       Node parent = thisNode.getParentNode();
 
       switch (scheme) {
-      case CHILD:
-        Node[] children = adoptAsChild(thisNode, otherNode);
-        if (children.length == 1 && otherNode == children[0]) {
+        case CHILD:
+          Node[] children = adoptAsChild(thisNode, otherNode);
+          if (children.length == 1 && otherNode == children[0]) {
+            break;
+          } else {
+            nodeOrTags = nodeArrayToRubyArray(context.runtime, children);
+          }
+          break;
+        case PREV_SIBLING:
+          adoptAsPrevSibling(context, parent, thisNode, otherNode);
+          break;
+        case NEXT_SIBLING:
+          adoptAsNextSibling(context, parent, thisNode, otherNode);
+          break;
+        case REPLACEMENT:
+          adoptAsReplacement(context, parent, thisNode, otherNode);
           break;
-        } else {
-          nodeOrTags = nodeArrayToRubyArray(context.runtime, children);
-        }
-        break;
-      case PREV_SIBLING:
-        adoptAsPrevSibling(context, parent, thisNode, otherNode);
-        break;
-      case NEXT_SIBLING:
-        adoptAsNextSibling(context, parent, thisNode, otherNode);
-        break;
-      case REPLACEMENT:
-        adoptAsReplacement(context, parent, thisNode, otherNode);
-        break;
       }
     } catch (Exception e) {
       throw context.runtime.newRuntimeError(e.toString());
@@ -1743,24 +1772,11 @@ public class XmlNode extends RubyObject
       e.appendChild(otherNode);
       otherNode = e;
     } else {
-      addNamespaceURIIfNeeded(otherNode);
       parent.appendChild(otherNode);
     }
     return new Node[] { otherNode };
   }
 
-  private void
-  addNamespaceURIIfNeeded(Node child)
-  {
-    if (this instanceof XmlDocumentFragment && ((XmlDocumentFragment) this).getFragmentContext() != null) {
-      XmlElement fragmentContext = ((XmlDocumentFragment) this).getFragmentContext();
-      String namespace_uri = fragmentContext.node.getNamespaceURI();
-      if (namespace_uri != null && namespace_uri.length() > 0) {
-        NokogiriHelpers.renameNode(child, namespace_uri, child.getNodeName());
-      }
-    }
-  }
-
   protected void
   adoptAsPrevSibling(ThreadContext context,
                      Node parent,
