@@ -371,9 +371,12 @@ new (VALUE klass, VALUE nodeobj)
   xmlXPathContextPtr ctx;
   VALUE self;
 
-  Data_Get_Struct(nodeobj, xmlNode, node);
+  Noko_Node_Get_Struct(nodeobj, xmlNode, node);
 
+#if LIBXML_VERSION < 21000
+  /* deprecated in 40483d0 */
   xmlXPathInit();
+#endif
 
   ctx = xmlXPathNewContext(node->doc);
   ctx->node = node;
