@@ -1,4 +1,5 @@
 # frozen_string_literal: true
+
 #
 # Some environment variables that are used to configure the test suite:
 # - NOKOGIRI_TEST_FAIL_FAST: if set to anything, emit test failure messages immediately upon failure
@@ -6,33 +7,37 @@
 #   - "normal" - normal GC functionality
 #   - "minor" - force a minor GC cycle after each test
 #   - "major" (default for Rubies without compaction) - force a major GC cycle after each test
-#   - "compact" (default for Rubies with compaction) - force a major GC and a compaction after each test
-#   - "verify" - force a major GC, a compaction, and verify references after each test
+#   - "compact" (default for Rubies with compaction) - force a major GC after each test and GC compaction after every 20 tests
+#   - "verify" - force a major GC after each test and verify references-after-compaction after every 20 tests
 #   - "stress" - run tests with GC.stress set to true
 # - NOKOGIRI_GC: read more in test/test_memory_leak.rb
 #
-require 'simplecov'
+
+# make sure we do one final major before the process exits (for valgrind)
+at_exit { GC.start(full_mark: true) } unless ::RUBY_PLATFORM == "java"
+
+require "simplecov"
 SimpleCov.start do
   add_filter "/test/"
 end
 
 $VERBOSE = true
 
-require 'minitest/autorun'
-require 'minitest/reporters'
+require "minitest/autorun"
+require "minitest/reporters"
 NOKOGIRI_MINITEST_REPORTERS_OPTIONS = { color: true, slow_count: 10, detailed_skip: false }
 NOKOGIRI_MINITEST_REPORTERS_OPTIONS[:fast_fail] = true if ENV["NOKOGIRI_TEST_FAIL_FAST"]
 puts "Minitest::Reporters options: #{NOKOGIRI_MINITEST_REPORTERS_OPTIONS}"
 Minitest::Reporters.use!(Minitest::Reporters::DefaultReporter.new(NOKOGIRI_MINITEST_REPORTERS_OPTIONS))
 
-require 'fileutils'
-require 'tempfile'
-require 'pp'
-require 'yaml'
+require "fileutils"
+require "tempfile"
+require "pp"
+require "yaml"
 
-require 'nokogiri'
+require "nokogiri"
 
-if ENV['TEST_NOKOGIRI_WITH_LIBXML_RUBY']
+if ENV["TEST_NOKOGIRI_WITH_LIBXML_RUBY"]
   #
   #  if you'd like to test with the libxml-ruby gem loaded, it's
   #  recommended that you set
@@ -42,7 +47,7 @@ if ENV['TEST_NOKOGIRI_WITH_LIBXML_RUBY']
   #  which will a) bundle that gem, and b) set the appropriate env var to
   #  trigger this block
   #
-  require 'libxml'
+  require "libxml"
   warn "#{__FILE__}:#{__LINE__}: loaded libxml-ruby '#{LibXML::XML::VERSION}'"
 end
 
@@ -51,35 +56,35 @@ warn Nokogiri::VERSION_INFO.to_yaml
 
 module Nokogiri
   class TestCase < MiniTest::Spec
-    ASSETS_DIR           = File.expand_path(File.join(File.dirname(__FILE__), 'files'))
-    ADDRESS_SCHEMA_FILE  = File.join(ASSETS_DIR, 'address_book.rlx')
-    ADDRESS_XML_FILE     = File.join(ASSETS_DIR, 'address_book.xml')
-    ENCODING_HTML_FILE   = File.join(ASSETS_DIR, 'encoding.html')
-    ENCODING_XHTML_FILE  = File.join(ASSETS_DIR, 'encoding.xhtml')
-    EXML_FILE            = File.join(ASSETS_DIR, 'exslt.xml')
-    EXSLT_FILE           = File.join(ASSETS_DIR, 'exslt.xslt')
-    HTML_FILE            = File.join(ASSETS_DIR, 'tlm.html')
-    METACHARSET_FILE     = File.join(ASSETS_DIR, 'metacharset.html')
-    NICH_FILE            = File.join(ASSETS_DIR, '2ch.html')
-    NOENCODING_FILE      = File.join(ASSETS_DIR, 'noencoding.html')
-    PO_SCHEMA_FILE       = File.join(ASSETS_DIR, 'po.xsd')
-    PO_XML_FILE          = File.join(ASSETS_DIR, 'po.xml')
-    SHIFT_JIS_HTML       = File.join(ASSETS_DIR, 'shift_jis.html')
-    SHIFT_JIS_NO_CHARSET = File.join(ASSETS_DIR, 'shift_jis_no_charset.html')
-    SHIFT_JIS_XML        = File.join(ASSETS_DIR, 'shift_jis.xml')
-    SNUGGLES_FILE        = File.join(ASSETS_DIR, 'snuggles.xml')
-    XML_FILE             = File.join(ASSETS_DIR, 'staff.xml')
-    XML_XINCLUDE_FILE    = File.join(ASSETS_DIR, 'xinclude.xml')
-    XML_ATOM_FILE        = File.join(ASSETS_DIR, 'atom.xml')
-    XSLT_FILE            = File.join(ASSETS_DIR, 'staff.xslt')
-    XPATH_FILE           = File.join(ASSETS_DIR, 'slow-xpath.xml')
+    ASSETS_DIR           = File.expand_path(File.join(File.dirname(__FILE__), "files"))
+    ADDRESS_SCHEMA_FILE  = File.join(ASSETS_DIR, "address_book.rlx")
+    ADDRESS_XML_FILE     = File.join(ASSETS_DIR, "address_book.xml")
+    ENCODING_HTML_FILE   = File.join(ASSETS_DIR, "encoding.html")
+    ENCODING_XHTML_FILE  = File.join(ASSETS_DIR, "encoding.xhtml")
+    EXML_FILE            = File.join(ASSETS_DIR, "exslt.xml")
+    EXSLT_FILE           = File.join(ASSETS_DIR, "exslt.xslt")
+    HTML_FILE            = File.join(ASSETS_DIR, "tlm.html")
+    METACHARSET_FILE     = File.join(ASSETS_DIR, "metacharset.html")
+    NICH_FILE            = File.join(ASSETS_DIR, "2ch.html")
+    NOENCODING_FILE      = File.join(ASSETS_DIR, "noencoding.html")
+    PO_SCHEMA_FILE       = File.join(ASSETS_DIR, "po.xsd")
+    PO_XML_FILE          = File.join(ASSETS_DIR, "po.xml")
+    SHIFT_JIS_HTML       = File.join(ASSETS_DIR, "shift_jis.html")
+    SHIFT_JIS_NO_CHARSET = File.join(ASSETS_DIR, "shift_jis_no_charset.html")
+    SHIFT_JIS_XML        = File.join(ASSETS_DIR, "shift_jis.xml")
+    SNUGGLES_FILE        = File.join(ASSETS_DIR, "snuggles.xml")
+    XML_FILE             = File.join(ASSETS_DIR, "staff.xml")
+    XML_XINCLUDE_FILE    = File.join(ASSETS_DIR, "xinclude.xml")
+    XML_ATOM_FILE        = File.join(ASSETS_DIR, "atom.xml")
+    XSLT_FILE            = File.join(ASSETS_DIR, "staff.xslt")
+    XPATH_FILE           = File.join(ASSETS_DIR, "slow-xpath.xml")
 
     unless Nokogiri.jruby?
-      GC_LEVEL = if ["stress", "major", "minor", "normal"].include?(ENV['NOKOGIRI_TEST_GC_LEVEL'])
-        ENV['NOKOGIRI_TEST_GC_LEVEL']
-      elsif (ENV['NOKOGIRI_TEST_GC_LEVEL'] == "compact") && defined?(GC.compact)
+      GC_LEVEL = if ["stress", "major", "minor", "normal"].include?(ENV["NOKOGIRI_TEST_GC_LEVEL"])
+        ENV["NOKOGIRI_TEST_GC_LEVEL"]
+      elsif (ENV["NOKOGIRI_TEST_GC_LEVEL"] == "compact") && defined?(GC.compact)
         "compact"
-      elsif (ENV['NOKOGIRI_TEST_GC_LEVEL'] == "verify") && defined?(GC.verify_compaction_references)
+      elsif (ENV["NOKOGIRI_TEST_GC_LEVEL"] == "verify") && defined?(GC.verify_compaction_references)
         "verify"
       else
         defined?(GC.compact) ? "compact" : "major"
@@ -87,7 +92,11 @@ module Nokogiri
       warn "#{__FILE__}:#{__LINE__}: NOKOGIRI_TEST_GC_LEVEL: #{GC_LEVEL}"
     end
 
+    COMPACT_EVERY = 20
+    @@test_count = 0 # rubocop:disable Style/ClassVars
+
     def setup
+      @@test_count += 1 # rubocop:disable Style/ClassVars
       if Nokogiri.uses_libxml?
         @fake_error_handler_called = false
         Nokogiri::Test.__foreign_error_handler do
@@ -110,10 +119,16 @@ module Nokogiri
         when "major"
           GC.start(full_mark: true)
         when "compact"
-          GC.compact
+          if @@test_count % COMPACT_EVERY == 0
+            GC.compact
+          else
+            GC.start(full_mark: true)
+          end
         when "verify"
-          # https://alanwu.space/post/check-compaction/
-          GC.verify_compaction_references(double_heap: true, toward: :empty)
+          if @@test_count % COMPACT_EVERY == 0
+            # https://alanwu.space/post/check-compaction/
+            GC.verify_compaction_references(double_heap: true, toward: :empty)
+          end
           GC.start(full_mark: true)
         when "stress"
           GC.stress = false
@@ -138,11 +153,22 @@ module Nokogiri
       end
     end
 
-    def skip_unless_libxml2(msg="this test should only run with libxml2")
+    def skip_unless_libxml2(msg = "this test should only run with libxml2")
       skip(msg) unless Nokogiri.uses_libxml?
     end
 
-    def skip_unless_jruby(msg="this test should only run with jruby")
+    def skip_unless_libxml2_patch(patch_name)
+      patch_dir = File.join(__dir__, "..", "patches", "libxml2")
+      if File.directory?(patch_dir) && !File.exist?(File.join(patch_dir, patch_name))
+        raise("checking for nonexistent patch file #{patch_name.inspect}")
+      end
+
+      unless Nokogiri.libxml2_patches.include?(patch_name)
+        skip("this test needs libxml2 patched with #{patch_name}")
+      end
+    end
+
+    def skip_unless_jruby(msg = "this test should only run with jruby")
       skip(msg) unless Nokogiri.jruby?
     end
 
@@ -160,9 +186,9 @@ module Nokogiri
       doc.traverse do |node|
         nodes << node if node.text? && node.blank?
       end
-      assert(nodes.length > 0)
+      refute_empty(nodes)
       nodes.each do |node|
-        len = node.content.gsub(/[\r\n]/, '').length
+        len = node.content.gsub(/[\r\n]/, "").length
         assert_equal(0, len % amount, message)
       end
     end
@@ -173,22 +199,29 @@ module Nokogiri
       document.decorate!
     end
 
-    #
-    #  Test::Unit backwards compatibility section
-    #
-    alias_method :assert_no_match, :refute_match
-    alias_method :assert_not_nil, :refute_nil
-    alias_method :assert_raise, :assert_raises
-    alias_method :assert_not_equal, :refute_equal
-
     def assert_not_send(send_ary, m = nil)
       recv, msg, *args = send_ary
       m = message(m) do
         "Expected #{mu_pp(recv)}.#{msg}(*#{mu_pp(args)}) to return false"
       end
-      assert(!recv.__send__(msg, *args), m)
+      refute(recv.__send__(msg, *args), m)
     end unless method_defined?(:assert_not_send)
 
+    def pending(msg)
+      begin
+        yield
+      rescue MiniTest::Assertion
+        skip("pending #{msg}")
+      end
+      flunk("pending test unexpectedly passed: #{msg}")
+    end
+
+    def pending_if(msg, pend_eh, &block)
+      return yield unless pend_eh
+
+      pending(msg, &block)
+    end
+
     def i_am_ruby_matching(gem_version_requirement_string)
       Gem::Requirement.new(gem_version_requirement_string).satisfied_by?(Gem::Version.new(RUBY_VERSION))
     end
@@ -201,6 +234,14 @@ module Nokogiri
       # https://stackoverflow.com/questions/365458/how-can-i-detect-if-a-program-is-running-from-within-valgrind/62364698#62364698
       ENV["LD_PRELOAD"] =~ /valgrind|vgpreload/
     end
+
+    def i_am_running_with_asan
+      # https://stackoverflow.com/questions/35012059/check-whether-sanitizer-like-addresssanitizer-is-active
+
+      %x"ldd #{Gem.ruby}".include?("libasan.so")
+    rescue
+      false
+    end
   end
 
   module SAX
@@ -293,6 +334,7 @@ module Nokogiri
         attr_reader :items
 
         def initialize
+          super
           # [
           #   [ :method_1, argument_1, ... ],
           #   [ :method_2, argument_2, ... ],
@@ -308,7 +350,7 @@ module Nokogiri
           :start_element_namespace, :end_element_namespace,
           :characters, :comment, :cdata_block,
           :processing_instruction,
-          :error, :warning
+          :error, :warning,
         ].each do |name|
           define_method name do |*arguments|
             @items << [name, *arguments]
