@@ -1,9 +1,10 @@
+# frozen_string_literal: true
+
 require "helper"
 
 module Nokogiri
   module XML
     module SAX
-
       # raises an exception when underlying parser
       # encounters an XML parsing error
       class ThrowingErrorDocument < Document
@@ -25,29 +26,18 @@ module Nokogiri
       end
 
       class TestErrorHandling < Nokogiri::SAX::TestCase
-        def setup
-          super
-          @error_parser = Parser.new(ThrowingErrorDocument.new)
-          @warning_parser = Parser.new(WarningErrorDocument.new)
-        end
-
         def test_error_throwing_document_raises_exception
-          begin
-            @error_parser.parse("<xml>") # no closing element
-            fail "#parse should not complete successfully when document #error throws exception"
-          rescue StandardError => e
-            assert_match(/parsing did not complete/, e.message)
+          error_parser = Parser.new(ThrowingErrorDocument.new)
+          e = assert_raises(StandardError) do
+            error_parser.parse("<xml>") # no closing element
           end
+          assert_match(/parsing did not complete/, e.message)
         end
 
         def test_warning_document_encounters_error_but_terminates_normally
-          begin
-            @warning_parser.parse("<xml>")
-            assert !@warning_parser.document.errors.empty?, "error collector did not collect an error"
-          rescue StandardError => e
-            warn(e)
-            fail '#parse should complete successfully unless document #error throws exception (#{e}'
-          end
+          warning_parser = Parser.new(WarningErrorDocument.new)
+          warning_parser.parse("<xml>")
+          refute_empty(warning_parser.document.errors, "error collector did not collect an error")
         end
       end
     end
