@@ -402,7 +402,7 @@ class Nokogiri::SAX::TestCase
     end
 
     it :test_large_cdata_is_handled do
-      skip("see #2132 and https://gitlab.gnome.org/GNOME/libxml2/-/issues/200") if Nokogiri.uses_libxml?("<=2.9.10")
+      skip("see #2132 and https://gitlab.gnome.org/GNOME/libxml2/-/issues/200") if Nokogiri::VersionInfo.instance.libxml2_using_system?
 
       template = <<~EOF
         <?xml version="1.0" encoding="UTF-8"?>
@@ -429,7 +429,12 @@ class Nokogiri::SAX::TestCase
       parser = Nokogiri::XML::SAX::Parser.new(handler)
       parser.parse(xml)
 
-      assert_predicate(handler.errors, :empty?)
+      if Nokogiri.uses_libxml?(">=2.10.3")
+        # CVE-2022-40303 https://gitlab.gnome.org/GNOME/libxml2/-/commit/c846986
+        assert_match(/CData section too big/, handler.errors.first)
+      else
+        assert_predicate(handler.errors, :empty?)
+      end
     end
 
     it "does not resolve entities by default" do
