@@ -13,7 +13,7 @@ module Nokogiri
       def test_cdata_node
         name = @xml.xpath("//employee[2]/name").first
         assert(cdata = name.children[1])
-        assert(cdata.cdata?)
+        assert_predicate(cdata, :cdata?)
         assert_equal("#cdata-section", cdata.name)
       end
 
