@@ -1,3 +1,5 @@
+# frozen_string_literal: true
+
 require "helper"
 
 module Nokogiri
@@ -9,22 +11,22 @@ module Nokogiri
       end
 
       def test_url
-        assert_equal 'UTF-8', @xml.url.encoding.name
+        assert_equal("UTF-8", @xml.url.encoding.name)
       end
 
       def test_encoding
-        assert_equal 'UTF-8', @xml.encoding.encoding.name
+        assert_equal("UTF-8", @xml.encoding.encoding.name)
       end
 
       def test_dotted_version
         skip_unless_libxml2
-        assert_equal "UTF-8", Nokogiri::LIBXML_COMPILED_VERSION.encoding.name
-        assert_equal "UTF-8", Nokogiri::LIBXSLT_COMPILED_VERSION.encoding.name
+        assert_equal("UTF-8", Nokogiri::LIBXML_COMPILED_VERSION.encoding.name)
+        assert_equal("UTF-8", Nokogiri::LIBXSLT_COMPILED_VERSION.encoding.name)
       end
 
       def test_empty_doc_encoding
-        encoding = 'US-ASCII'
-        assert_equal encoding, Nokogiri::XML(nil, nil, encoding).encoding
+        encoding = "US-ASCII"
+        assert_equal(encoding, Nokogiri::XML(nil, nil, encoding).encoding)
       end
     end
   end
